/**
 * @author Rafael Ostertag <rafael.ostertag@math.uzh.ch>
 */

/* global $ */
/* global console */
/* global Bloodhound */
/* global Math */

/* @depend Utils.js */

var QfqNS = QfqNS || {};

(function (n) {
    'use strict';

    n.TypeAhead = {};


    /**
     *
     * @param typeahead_endpoint the endpoint to be called
     * @constructor
     */
    n.TypeAhead.install = function (typeahead_endpoint, prefetch_endpoint) {

        $('.qfq-typeahead').each(function () {
            var $shadowElement, bloodhoundConfiguration;

            var $element = $(this);
            bloodhoundConfiguration = {
                datumTokenizer: Bloodhound.tokenizers.obj.whitespace('key', 'value'),
                queryTokenizer: Bloodhound.tokenizers.whitespace,
                identify: function (obj) {
                    return obj.key;
                },
                remote: {
                    url: n.TypeAhead.makeUrl(typeahead_endpoint, $element),
                    wildcard: '%QUERY'
                }
            };
            if ($element.val() !== '') {
                // We prefetch the value provided
                bloodhoundConfiguration.prefetch = n.TypeAhead.makePrefetchUrl(typeahead_endpoint, $element.val(), $element);
            }


            var suggestions = new Bloodhound(bloodhoundConfiguration);

            n.TypeAhead.makeShadowElement($element);

            $element.typeahead({
                    hint: n.TypeAhead.getHint($element),
                    highlight: n.TypeAhead.getHighlight($element),
                    minLength: n.TypeAhead.getMinLength($element)
                },
                {
                    display: 'value',
                    source: suggestions,
                    limit: n.TypeAhead.getLimit($element),
                    templates: {
                        suggestion: function (obj) {
                            return "<div>" + n.TypeAhead.htmlEncode(obj.value) + "</div>";
                        },
                        notFound: function (obj) {
                            return "<div>'" + n.TypeAhead.htmlEncode(obj.query) + "' not found";
                        }
                    }
                });

            n.TypeAhead.fillTypeAheadFromShadowElement($element, suggestions);

            $element.bind('typeahead:select typeahead:autocomplete', function (event, suggestion) {
                var $shadowElement = n.TypeAhead.getShadowElement($(event.delegateTarget));
                $shadowElement.val(suggestion.key);
            });

            if (!!$element.data('typeahead-pedantic')) {
                $element.bind('typeahead:change', (function (bloodHound) {
                        return function (event) {
                            var $typeAhead = $(event.delegateTarget);
                            var $shadowElement = n.TypeAhead.getShadowElement($typeAhead);
                            if ($shadowElement.val() === '') {
                                $typeAhead.typeahead('val', '');
                                $typeAhead.closest('form').change();
                            }
                        };
                    })(suggestions)
                );
            }
        });

    };

    n.TypeAhead.makeUrl = function (endpoint, element) {
        return endpoint + "?query=%QUERY" + "&sip=" + n.TypeAhead.getSip(element);
    };
    n.TypeAhead.makePrefetchUrl = function (endpoint, prefetchKey, element) {
        return endpoint + "?prefetch=" + encodeURIComponent(prefetchKey) + "&sip=" + n.TypeAhead.getSip(element);
    };

    n.TypeAhead.getLimit = function ($element) {
        return $element.data('typeahead-limit');
    };

    n.TypeAhead.getSip = function ($element) {
        return $element.data('typeahead-sip');
    };

    n.TypeAhead.getName = function ($element) {
        return $element.attr('name');
    };

    n.TypeAhead.getValue = function ($element) {
        return $element.val();
    };

    n.TypeAhead.getMinLength = function ($element) {
        return $element.data('typeahead-minlength') || 2;
    };

    n.TypeAhead.getHighlight = function ($element) {
        return $element.data('typeahead-highlight') || true;
    };

    n.TypeAhead.getHint = function ($element) {
        return $element.data('typeahead-hint') || true;
    };

    n.TypeAhead.htmlEncode = function (value) {
        return $('<div/>').text(value).html();
    };

    n.TypeAhead.makeShadowElement = function ($element) {
        var $parent, inputName, inputValue, uniqueId, $shadowElement;

        $parent = $element.parent();
        inputName = $element.attr('name');
        $element.removeAttr('name');

        inputValue = $element.val();

        $shadowElement = $('<input>')
            .attr('type', 'hidden')
            .attr('name', inputName)
            .val(inputValue);

        $element.data('shadow-element', $shadowElement);

        $parent.append($shadowElement);

        return $shadowElement;
    };

    n.TypeAhead.getShadowElement = function ($element) {
        return $element.data('shadow-element');
    };

    n.TypeAhead.fillTypeAheadFromShadowElement = function ($element, bloodhound) {
        var results;
        var $shadowElement = n.TypeAhead.getShadowElement($element);
        var key = $shadowElement.val();
        if (key === '') {
            return;
        }

        results = bloodhound.get(key);
        if (results.length === 0) {
            return;
        }
        $element.typeahead('val', results[0].value);
    };
})(QfqNS);

